#ifndef __NSG_HDLC_H
#define __NSG_HDLC_H

#include <linux/nsgrouter.h>

#define NSG_PORT_NEW_HOOK 0
#define NSG_PORT_DELETE_HOOK 1
#define NSG_ROUTER_PORT_DIR "/proc/net/"NSG_DIR_NAME"/port"


/*
 * Data types for configuration structure description tables.
 */
#define	DTYPE_INT	1
#define	DTYPE_UINT	2
#define	DTYPE_LONG	3
#define	DTYPE_ULONG	4
#define	DTYPE_SHORT	5
#define	DTYPE_USHORT	6
#define	DTYPE_CHAR	7
#define	DTYPE_UCHAR	8
#define	DTYPE_PTR	9
#define	DTYPE_STR	10
#define	DTYPE_FILENAME	11
#define	DTYPE_INET	12
#define DTYPE_CONFIG    13

/* port node parameters */
#define ENCAPSULATION_STR "encapsulation"
#define CISCO_HDLC_STR    "cisco-hdlc"
#define FRAME_RELAY_STR   "frame-relay"
#define PHY_INTERFACE_STR "physical"
#define CLOCK_STR         "clock"
#define BAUDRATE_STR      "baudrate"


/* port-cisco_hdlc node parameters */
#define KEEPALIVE_STR     "keepalive"
#define TIMEOUT_STR       "timeout"

/* port-frame_relay node parameters */
#define DLCI_STR          "dlci"
#define LMI_STR           "lmi"
#define INTF_TYPE_STR     "intf-type"
#define T391_STR          "t391"
#define T392_STR          "t392"
#define N391_STR          "n391"
#define N392_STR          "n392"
#define N393_STR          "n393"

/* port-interface node parameters */
#define V24_STR  "v24"
#define V35_STR  "v35"
#define SDSL_STR "sdsl"

/* port-clock node parameters */
#define EXTERNAL_STR "external"
#define INTERNAL_STR "internal"
#define MASTER_STR   "master"
#define SLAVE_STR    "slave"

/* port-fr-lmi values */
#define NONE_STR  "none"
#define ANSI_STR  "ansi"
#define CCITT_STR "ccitt"

/* port-fr-intf_type value*/
#define DTE_STR "dte"
#define DCE_STR "dce"


/* nsg-port flags */
#define NSG_FLG_NOSHUTDOWN (1<<0)


struct nsg_port_t{
  char name[NSG_IFNAME_SZ+1];
  uint flags;
  struct list *channel;
  nsgdev_conf_t conf;
} nsg_port_t;

struct nsg_channel_t {
  struct nsg_port_t *portp;
  uint flags;
  nsgif_conf_t conf;
} nsg_channel_t;

typedef struct nsg_value {
  char *name;
  uint val;
} nsg_value_t;

typedef struct nsg_port_config_t {
  char *keyword;
  int  dtype;
  uint offset;
  char *default_value;
  void *value;
  char *cond;
  void (*set_hook)(char *, struct nsg_port_config_t *, struct nsg_port_t *);  
} nsg_port_config_t;


void set_encap_param_hook(char *, nsg_port_config_t *, struct nsg_port_t *);

#endif
